; Testovaci program pro A/D prevodnik PIC16C71x.
; Vysledky prevodu se kazdych 40ms prepisuji do registru ad0 az ad3.
;***************************************************************************
         list   p=16c711
; frekvence vnejsiho oscilatoru je 3.2768MHz => instrukcni cyklus = 1.22us
; 1.22us * 256 (rtcc) * 128 (prescaler) = 40ms
;***************************************************************************
; Zapojeni pinu
;   RA0 A/D prevodnik ad0                               (in)
;   RA1 A/D prevodnik ad1                               (in)
;   RA2 A/D prevodnik ad2                               (in)
;   RA3 A/D prevodnik ad2                               (in)
;   RA4 nepouzito                                       (out)
;   RB0 az RB7 nepouzito                                (out)
;***************************************************************************
rtcc    equ	1

status  equ     3
#define carry   status,0
#define zero    status,2
#define rp0     status,5

porta	equ	5

portb	equ	6

adcon   equ     8          ; adcon0/adcon1 se rozlisuje pomoci rp0
#define go      adcon,2    ; bit 2 registru adcon0 (zahajeni prevodu)
#define chs0    adcon,3    ; bit 3 registru adcon0
#define chs1    adcon,4    ; bit 4 registru adcon0 (vyber prevodniku)

adres	equ	9

intcon	equ	0x0B

; uzivatelske definice symbolu
citac   equ     0x0C       ; citac pro prodlevu 36us
ad0     equ     0x10       ; vysledek prevodu pro prevodnik 0
ad1     equ     0x11       ; vysledek prevodu pro prevodnik 1
ad2     equ     0x12       ; vysledek prevodu pro prevodnik 2
ad3     equ     0x13       ; vysledek prevodu pro prevodnik 3


        org     0   ; tady zacina PIC po resetu
;***************************************************************************
start   call    init            ; inicializace
loop    call    prev0           ; prevodnik 0
	call    prev1           ; prevodnik 1
	call    prev2           ; prevodnik 2
	call    prev3           ; prevodnik 3
	call    _40ms           ; cekej na vyprseni 40ms
	goto    loop            ; a porad dokola
;***************************************************************************
init    movlw   b'00000000'
	movwf   intcon          ; preruseni zakazano
	clrf    rtcc            ; nulovani rtcc
	movlw   b'10000110'     ; pull-up na portu B disable, rtcc buzen
	option                  ; interne, preddelic 1:128, tedy cita na 40ms
	clrf    porta           ; nulovani portu A
	clrf    portb           ; nulovani portu B
	movlw   b'11101111'
	tris    porta           ; port A jako vstup krome RA4
	movlw   b'00000000'
	tris    portb           ; port B jako vystup
	bsf     rp0             ; stranka 1
	movlw   b'00000000'
	movwf   adcon           ; RA0 - RA3 jako analogove vstupy (adcon1)
	bcf     rp0             ; zpet na stranku 0
	movlw   b'01000001'
	movwf   adcon ; tad=8*tosc, tj. 2.44us, A/D zapnut, prodleva (adcon0)
; na sampl a prevod tedy bohate staci 36us
	retlw   00h             ; a konec
;***************************************************************************
prev0   bcf     chs1
	bcf     chs0            ; prevodnik 0
	call    _36us           ; prodleva 36us na sampl
	bsf     go              ; zahajeni prevodu
	call    _36us           ; prodleva 36us na prevod
	movf    adres, w
	movwf   ad0             ; prepis vysledku do registru ad0
	retlw   00h             ; a konec
;***************************************************************************
prev1   bcf     chs1
	bsf     chs0            ; prevodnik 1
	call    _36us           ; prodleva 36us na sampl
	bsf     go              ; zahajeni prevodu
	call    _36us           ; prodleva 36us na prevod
	movf    adres, w
	movwf   ad1             ; prepis vysledku do registru ad1
	retlw   00h             ; a konec
;***************************************************************************
prev2   bsf     chs1
	bcf     chs0            ; prevodnik 2
	call    _36us           ; prodleva 36us na sampl
	bsf     go              ; zahajeni prevodu
	call    _36us           ; prodleva 36us na prevod
	movf    adres, w
	movwf   ad2             ; prepis vysledku do registru ad2
	retlw   00h             ; a konec
;***************************************************************************
prev3   bsf     chs1
	bsf     chs0            ; prevodnik 3
	call    _36us           ; prodleva 36us na sampl
	bsf     go              ; zahajeni prevodu
	call    _36us           ; prodleva 36us na prevod
	movf    adres, w
	movwf   ad3             ; prepis vysledku do registru ad3
	retlw   00h             ; a konec
;***************************************************************************
; Rutina zajisti cekani 36us.
_36us   movlw   .9
	movwf   citac
_36us_a decfsz  citac, f
	goto    _36us_a
	retlw   00h             ; konec
;***************************************************************************
; Rutina zajisti cekani 40ms (ne od okamziku volani rutiny, ale od posledniho
; preteceni rtcc).
_40ms   comf    rtcc, w
	btfss   zero            ; rtcc = 0ffh? (Z=1?)
	goto    _40ms           ; ne  => cekame
; ano => testujeme preteceni
_40ms_a btfsc   rtcc, 0         ; rtcc pretekl? (bit0=0?)
	goto    _40ms_a         ; ne  => cekame na preteceni
	retlw   00h             ; ano => konec
;***************************************************************************
 end
